package eap.freeciv;

import java.io.*;

public class BinaryField extends Field {

String name;
int size;
String size_field;

/****************************************************************************
*
****************************************************************************/
public void setName(String name) { this.name = name; }


/****************************************************************************
*
****************************************************************************/
public void addIndex(int index) {

    name = name + "["+index+"]";
}

/****************************************************************************
*
****************************************************************************/
public void removeIndex() {

    int bracket = name.lastIndexOf('[');
    if(bracket==-1) return;
    name = name.substring(0,bracket);
}

/****************************************************************************
*
****************************************************************************/
public boolean isSet(Packet packet) {

    return packet.hasValue(name);
}

/****************************************************************************
*
****************************************************************************/
public void setSize(String size) {

    this.size = -1;
    size_field = null;

    try { this.size = Integer.parseInt(size); }
    catch(NumberFormatException e) {
        size_field = size;
    }
} // end of setRepeat method

/****************************************************************************
*
****************************************************************************/
public void read(Packet packet, PacketSource source) throws IOException {

    int nbytes = size;
    if(size_field != null) {
        /******************************************
        * get the size from a field in the packet *
        ******************************************/
        nbytes = ((Number)packet.getValue(size_field)).intValue();
    }

    byte[] data = new byte[nbytes];
    source.readBytes(data);

    packet.setValue(name, data);
    



} // end of read method

/****************************************************************************
*
****************************************************************************/
public void write(Packet packet, OutputStream out) throws IOException {

    try { 
        byte[] data = (byte[])packet.getValue(name);
        if(data == null) throw new IOException("Could not get "+name+" from "+packet);

        /*****************
        * check the size *
        *****************/
        if(size_field == null) {
            if(data.length != size) {
                throw new IOException(name+" in "+packet+" has length "+
                                      data.length+" instead of "+size);
            }
        } else {
            int nbytes = ((Number)packet.getValue(size_field)).intValue();
            if(data.length != nbytes) {
                throw new IOException("Size ("+data.length+" of "+name+" in "+
                                      packet+" does not match "+
                                      size_field+" = "+nbytes);
            }
        }
        
        /*****************
        * write the data *
        *****************/
        out.write(data);


    } catch(ClassCastException e) {
        throw new IOException(name+" is not a byte array in "+packet);
    }

} // end of write method

/****************************************************************************
*
****************************************************************************/
public int getSize(Packet packet) {

    if(size_field==null) return size;
    else                 return ((Number)packet.getValue(size_field)).intValue();
} // end of getSize method

} // end of BinaryField class
