package eap.freeciv;

import java.net.*;
import java.io.*;

/******************************************************************************
* Provides a means of establishing a {@link PacketConnection} to a server.
******************************************************************************/
public class Client implements PacketListener {

PacketConnection connection;


/****************************************************************************
* Returns the underlying PacketConnection.
****************************************************************************/
public PacketConnection getConnection() { return connection; }

/****************************************************************************
* Internally creates the {@link PacketConnection} connected to a server
* on a given host at a given port. By default freeciv servers listen to
* port 5555, and you can use "localhost" for a sever on your local machine.
****************************************************************************/
public void connectToServer(String host, int port) throws UnknownHostException, IOException {

    Socket socket = new Socket(host, port);
    connection = new PacketConnection(socket);

} // end of connectToServer method

/****************************************************************************
* Sends the given packet through the internal connection to the server.
* Since this class is a {@link PacketListener} you can link it to a
* {@link Server} to make a {@link PassThrough}
****************************************************************************/
public void receivePacket(Packet packet) {

    try { connection.sendPacket(packet); }
    catch(IOException e) { 
        System.err.println("Client could not send packet");
        System.err.println(e);
        System.exit(1);
    }
}



} // end of Client class
