package eap.freeciv;

import java.io.*;
import java.net.*;

/****************************************************************************
* This is a combination of {@link Client} and {@link Server} linked together
* so that when one receives a packet it passes it to the other.
* Thus a PassThrough can sit in between a "real" freeciv client and server.
* and monitor or modify the packets which pass between the two.
* The theory is that you could write a PassThrough which would automate
* some of the things you do on the client side by writing a PassThrough to
* handle them. The benefits are you could still use the standard client
* (saving alot of work making your own GUI) but you wouldn't have to touch
* any of the freeciv code.
****************************************************************************/
public class PassThrough {

Client client;
Server server;

/****************************************************************************
*
****************************************************************************/
public PassThrough(int server_port, String host, int client_port) 
       throws UnknownHostException, IOException {

    client = new Client();
    client.connectToServer(host, client_port);

    server = new Server(server_port);
    server.listenForClient();

    /***********************************************
    * connect the client and server to one another *
    ***********************************************/
    client.getConnection().addPacketListener(server);
    server.getConnection().addPacketListener(client);


} // end of constructor

/****************************************************************************
*
****************************************************************************/
public void beginListening() {

    client.getConnection().beginListening();
    server.getConnection().beginListening();
}

/****************************************************************************
*
****************************************************************************/
public Client getClient() { return client; }
public Server getServer() { return server; }

/****************************************************************************
*
****************************************************************************/
public static void main(String[] args) throws Exception {

    PassThrough pass = new PassThrough(4444,"localhost", 5555);

    pass.getClient().getConnection().addPacketListener(new PacketDumper("From Server"));
    pass.getServer().getConnection().addPacketListener(new PacketDumper("From Client"));
    
    Player player = new Player();
    pass.getClient().getConnection().addPacketListener(player);

    pass.beginListening();

} // end of main method
} // end of PassThrough class
