package eap.freeciv;

import java.io.*;
import java.net.*;
import java.util.*;

/****************************************************************************
* A server listens for connections on a particular port and creates
* a {@link PacketConnection} to each client.
****************************************************************************/
public class Server implements PacketListener {

ServerSocket server_socket;
List connections;


/****************************************************************************
* Create a new Server which will listen for packets on a given port.
****************************************************************************/
public Server(int port) throws IOException {

    server_socket = new ServerSocket(port);
    connections = new ArrayList();
} // end of constructor

/****************************************************************************
* Returns the number of clients connected to this server.
****************************************************************************/
public int getClientCount() { return connections.size(); }

/****************************************************************************
* Return the connection to a given client
****************************************************************************/
public PacketConnection getConnection(int index) {

    return (PacketConnection)connections.get(index);

} // end of getConnection method

/****************************************************************************
* Return the connection to the first client connected to this server.
* this is mostly for convenience for servers with only one client.
****************************************************************************/
public PacketConnection getConnection() { return getConnection(0); }

/****************************************************************************
* This method blocks until a client connects to this server. You need to 
* call this method every time you want to accept a new connection.
* A full server implementation would include a thread which would continually
* liten for connections, but I'm not really interested in that. Mostly I 
* wanted a server so that I could make a {@link PassThrough} that will sit
* in between a "real" server and client. Such a server would usually only 
* have one client.
****************************************************************************/
public void listenForClient() throws IOException {

    Socket socket = server_socket.accept();
    
    PacketConnection connection = new PacketConnection(socket);
    connections.add(connection);

} // end of listenForClient method

/****************************************************************************
* broadcast the given packet to all the clients
* This method implements the {@link PacketListener} interface. The idea of
* this is to link a client and server together to make a {@link PassThrough}.
****************************************************************************/
public void receivePacket(Packet packet) {

    for(Iterator it = connections.iterator(); it.hasNext(); ) {
        PacketConnection connection = (PacketConnection)it.next();

        try { connection.sendPacket(packet); }
        catch(IOException e) {
            System.err.println("Server could not send packet");
            System.err.println(e);
            System.exit(1);
        }
    }


} // end of receivePacket method

} // end of Server class
