package eap.freeciv;

import java.io.*;

public class BitField extends Field {

String[] names;

/****************************************************************************
*
****************************************************************************/
public BitField() {
    names = new String[8];
}

/****************************************************************************
*
****************************************************************************/
public void setName(String name, int bit) {

    names[bit] = name;
}

/****************************************************************************
*
****************************************************************************/
public void addIndex(int index) {

    for(int i=0; i<names.length; ++i) {
        if(names[i] == null ) continue;

        names[i] = names[i] + "["+index+"]";
    }
} // end of addIndex method

/****************************************************************************
*
****************************************************************************/
public void removeIndex() {

    for(int i=0; i<names.length; ++i) {
        if(names[i] == null ) continue;

        int bracket = names[i].lastIndexOf('[');
        if(bracket==-1) continue;

        names[i] = names[i].substring(0,bracket);
    }
} // end of removeIndex method

/****************************************************************************
* returns true if all the bit fields are set
****************************************************************************/
public boolean isSet(Packet packet) {

    for(int i=0; i<names.length; ++i) {
        if(names[i] == null ) continue;

        if(!packet.hasValue(names[i]) ) return false;
    }
    
    return true;


} // end of isSet method

/****************************************************************************
*
****************************************************************************/
public void read(Packet packet, PacketSource source) throws IOException {

    int field = source.readByte();

    int mask = 1;
    for(int i=0; i<8; ++i, mask <<=1) {
        if(names[i]==null) continue;

        boolean value = (field&mask) != 0;
        packet.setValue(names[i], new Boolean(value) );
    }



} // end of read method

/****************************************************************************
*
****************************************************************************/
public void write(Packet packet, OutputStream out) throws IOException {

    int field = 0;

    int mask = 1;
    for(int i=0; i<8; ++i, mask <<=1) {
        if(names[i]==null) continue;
        
        try {
            boolean value = ((Boolean)packet.getValue(names[i])).booleanValue();
            if(value) field |= mask;
        } catch(Exception e) {
            throw new IOException("Could not get a boolean for "+names[i]);
        }
    } // end of loop over bits
    
    out.write(field);


} // end of write method

/****************************************************************************
*
****************************************************************************/
public int getSize(Packet packet) {
    return 1;
}

} // end of BitField class
