package eap.freeciv;

import java.io.*;

/**************************************************************************
*
**************************************************************************/
public class SignedIntField extends SimpleField {

int nbytes;

/**************************************************************************
*
**************************************************************************/
public SignedIntField(int nbytes) {
    this.nbytes = nbytes;
}


/**************************************************************************
*
**************************************************************************/
public void readValue(Packet packet, PacketSource source) throws IOException {

    /************************************************************
    * handle the first byte specially, so we can determine the
    * sign and know how to initialize the value 
    ************************************************************/
    int first = source.readByte();
    
    long value;
    if((first & 128)==0) value = 0l;
    else                 value = Long.MIN_VALUE;

    value <<=8;
    value |= first;

    /*********************************
    * now read the rest of the bytes *
    *********************************/
    for(int i=1; i<nbytes; ++i) {
        value <<=8;
        value |= source.readByte();
    }
    
   // System.out.println("    reading sint "+name+" = "+value);

    if(scale!= 1.0) packet.setValue(name, new Double(scale*(double)value) );
    else            packet.setValue(name, new Long(value) );


} // end of read method

/**************************************************************************
*
**************************************************************************/
public void writeValue(Object object, OutputStream out) throws IOException {

    /********************************
    * get the value from the packet *
    ********************************/
    long value;
    try {
        Number number = (Number)object;

        if(scale==1.0) value = number.longValue();
        else           value = (long)(number.doubleValue()/scale);
    } catch(ClassCastException e) {
        throw new IOException(name+" = "+object+" is not a number\n");
    }

    /******************
    * write the bytes *
    ******************/
    for(int i=nbytes-1; i>=0; --i) {
        out.write((int)((value >> (i*8)) & 255));
    }

} // end of writeValue method

/****************************************************************************
*
****************************************************************************/
public int getValueSize(Packet packet) {

    return nbytes;

} // end of getValueSize method

} // end of SignedIntegerField inner class
